<?php

namespace Drupal\beehotel_pricealterator;

use Drupal\Core\Session\AccountInterface;

/**
 * Class Alter in charge of the price alteration.
 *
 * @package Drupal\beehotel_pricealterator\Services
 */
class Alter {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Alter constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   */
  public function __construct(AccountInterface $currentUser) {
    $this->currentUser = $currentUser;
  }

  /**
   * Contains the alteration flow.
   */
  public function alter($data) {

    $tempstore = \Drupal::service('tempstore.private')->get('beehotel_pricealterator');

    // 1. Get content.
    // 1a. Base Table with base price
    $basetable = \Drupal::service('beehotel_pricealterator.prealter')->baseTable($data);

    // 1d. Available Alterators
    $pricealterator = \Drupal::service('plugin.manager.beehotel.pricealterator');
    $alterators = $pricealterator->sortedAlterators();

    // 1e. Enabled Alterators
    $alterators = $this->CheckStatus($alterators);
    $tempstore->set('beehotel_pricealterator_log', $alterators);

    /* 2. Loop per night.
     * Drupal Commerce calls Resolver once per order item (not once per night)
     * We loop nights here, providing an average price
     * to be multiplied by items (nights).
     * @todo keep track of every night price into the $data array.
     */
    for ($n = 0; $n < $data['norm']['dates_from_search_form']['days']; $n++) {
      $data['tmp'] = NULL;
      $data['tmp']['night_timestamp'] =
        $data['norm']['dates_from_search_form']['checkin']['timestamp'] + (24 * 3600 * $n);
      $data['tmp']['night_date'] = date('Y-m-d', $data['tmp']['night_timestamp']);

      foreach ($alterators as $a) {
        $alterator = new $a['class']([], [], [], []);

        $data = $alterator->alter($data, $basetable);
        if (isset($data['tmp']['price'])) {
          $price[$data['tmp']['night_timestamp']] = $data['tmp']['price'];
        }
      }
    }

    $price = array_filter($price);
    if (count($price)) {
      $average = array_sum($price) / count($price);
    }

    $data['amount'] = $average;
    return $data;
  }

  /**
   * Validate data.
   *
   * Check name change to lowerCamel format.
   */
  private function checkStatus($alterators) {
    $a = [];
    // @todo move this into yml config.
    $legenda = [
      0 => "disabled",
      1 => "enabled",
      2 => "work in progress",
    ];
    foreach ($alterators as $item) {
      if ($item['status'] == 1) {
        $a[] = $item;
      }
    }
    return $a;
  }

  /**
   * How to send this output to the debug block?
   *
   * @todo implement the feature.
   */
  public function beehotelLog($data, $context) {}

}
