<?php

namespace Drupal\beehotel_pricealterator\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Markup;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Provides route responses for the Example module.
 */
class Alterators extends ControllerBase {

  /**
   * Returns a list of ptice alterators.
   *
   * @return array
   *   A simple renderable array.
   */
  public function alteratorsList() {

    $pricealterator = \Drupal::service('plugin.manager.beehotel.pricealterator');
    $alterators = $pricealterator->sortedAlterators();
    $route_provider = \Drupal::service('router.route_provider');

    $rows = [];

    foreach ($alterators as $k => $a) {

      $settings = "";

      // @todo replace this with https://www.drupal.org/project/drupal/issues/3029545
      $route = 'beehotel_pricealterators.' . strtolower($k) . '_settings';
      $exists = count($route_provider->getRoutesByNames([$route])) === 1;

      if ($exists == 1) {
        $url = Url::fromRoute('beehotel_pricealterators.' . strtolower($k) . '_settings');
        $settings = Link::fromTextAndUrl(t("Settings"), $url)->toString();
      }

      $config = \Drupal::config('beehotel_pricealterators.' . $a['id'] . '.settings');
      $enabled = $config->get('status');

      if ($a['status'] == 0) {
        $a['status'] = Markup::create("<span class='disabled'>NA</span>");
      }
      elseif ($a['status'] == 1 && $config->get('status') == 1) {
        $a['status'] = Markup::create("<span class='enabled'>ON</span>");
      }
      elseif ($a['type'] == "optional" && $a['status'] == 1 && $config->get('status') == 0) {
        $a['status'] = Markup::create("<span class='disabled'>OFF</span>");
      }
      elseif ($a['status'] == 1) {
        $a['status'] = Markup::create("<span class='enabled'>AV</span>");
      }

      $mandatory = ($a['type'] == 'mandatory') ? '<span class="mandatory">*</span>' : '';
      $class = $a['class'];

      $rows[] = [
        Markup::create('<strong>' . $a['id'] . '</strong>' . $mandatory),
        $a['description'],
        $a['weight'],
        $a['status'],
        $settings,
      ];

    }

    $header = [
      'title' => t('Id'),
      'description' => t('Description'),
      'weight' => t('Weight'),
      'status' => t('Status'),
      'settings' => t('Settings'),
    ];

    $url = Url::fromUri('https://www.drupal.org/node/3333744');
    $doc = Link::fromTextAndUrl(t("how to code your custom price alterators"), $url)->toString();

    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No content has been found.'),
      '#caption' => t('Price Alterators on this system'),
    ];

    $tmp = $this->t("Your own Algorithm to produce smart prices from rules you define");

    $title = "";

    $description = $this->t("<h5>The BEE Hotel Algorithm will produce smart prices for your Hotel from rules you define with Alterators.</h5>") . "<br>";
    $description .= $this->t("The price Algorithm is a chain of Alterators that dynamically produces prices for your hospitality business according to a logic you define.") . "<br>";
    $description .= $this->t("In this page you can enable and set up existing alterators provided by the <a target='_blank' href='https://www.drupal.org/project/bee_hotel'>BEE Hotel module</a>. If your business needs more specific Alterators, you can add your own.") . "<br>";
    $description .= $this->t("Read the price Algorithm Doc page at <a target='_blank' href='https://www.drupal.org/node/3354393'>https://www.drupal.org/node/3354393</a>") . "<br>";
    $description .= '<br/>For support, ask the <a href="https://www.drupal.org/community">developer community</a> or the <a href="https://www.drupal.org/u/afagioli">module author</a><br>';
    $description .= '<h4>Table legenda</h4>';
    $description .= '<ul>';
    $description .= '<li>Id: Alterator Id';
    $description .= '<li><span class="mandatory">*</span>: mandatory';
    $description .= '<li>Description a string to define what this Alterators does';
    $description .= '<li>Weight: used by Drupal to sort Alterators';
    $description .= '<li>Status:';
    $description .= '<ul>';
    $description .= '<li>NA: Not Available, by code';
    $description .= '<li>AV: Available, by code';
    $description .= '<li>OFF: switched off, click settings to enable';
    $description .= '<li>ON: switched on, click settings to disable';
    $description .= '</ul>';
    $description .= '<li>Settings: a link to the Alterator setup form';
    $description .= '</ul>';

    $details = [
      '#type' => 'details',
      '#title' => $tmp,
      '#description' => $description,
      '#open' => FALSE,
    ];

    return [
      '#type' => '#markup',
      '#markup' => $title . " " . render($details) . render($build),
    ];
  }

}
