<?php

namespace Drupal\beehotel_pricealterators\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure OneNightOnly Alterator.
 */
class OneNightOnly extends ConfigFormBase {

  /**
   * Reference to the Alterator (as plugin).
   *
   *   This value matches the ID in the @PriceAlterator annotation.
   */
  public function pluginId() {
    return 'OneNightOnly';
  }

  /**
   * {@inheritdoc}
   */
  public function configName() {
    return 'beehotel_pricealterators.' . $this->pluginId() . '.settings';
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return $this->configName();
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      $this->configName(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config($this->configName());

    $form = [
      '#type' => 'fieldset',
      '#title' => $this->t('One Night Only alterator'),
      '#collapsible' => FALSE,
    ];

    $form['status'] = [
      '#default_value' => $config->get('status'),
      '#type' => 'checkbox',
      '#title' => $this->t('Enable this alterator'),
      '#description' => $this->t('When on, this alterator will be applied the BEEHotel prices'),
    ];

    $tmp = $this->t("Bee Hotel units Prices will be altered as per this percentage value");

    $form['increase']['percentage'] = [
      '#default_value' => $config->get('percentage'),
      '#type' => 'range_slider',
      '#title' => 'Percentage alteration',
      '#min' => -100,
      '#max' => 100,
      '#step' => 1,
      '#description' => $tmp,
      '#data-orientation' => 'horizontal',
      '#output' => 'below',
      '#output__field_prefix' => '',
      '#output__field_suffix' => '%',
    ];

    $tmp = $this->t("<b>Important:</b> This fixed value overrides above percentage value. Set this value to 0 to apply the percentage value");

    $form['increase']['fixed'] = [
      '#default_value' => $config->get('fixed'),
      '#type' => 'textfield',
      '#title' => $this->t('Fixed value in to add to one night only reservations'),
      '#description' => $tmp,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $this->config($this->configName())
      ->set('status', $form_state->getValue('status'))
      ->set('percentage', $form_state->getValue('percentage'))
      ->set('fixed', $form_state->getValue('fixed'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
