<?php

namespace Drupal\beehotel_vertical\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * A form to define a time range for Vertical.
 */
class TimeRangeForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'beehotel_vertical_timerange';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return "";
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $request = \Drupal::request();
    $session = $request->getSession();
    $range = $session->get('beehotel_requested_range');

    $items = $this->timeranges();
    foreach ($items as $key => $item) {
      $ranges[$key] = render($item['label']);
    }

    $form['range'] = [
      '#title' => t('Time range'),
      '#description' => t("Which time range do want to see?"),
      '#type' => 'select',
      '#options' => $ranges,
      '#default_value' => $range,
    ];

    $form['actions']['#type'] = 'actions';

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Search'),
      '#button_type' => 'primary',
    ];
    $form['#theme'] = 'vertical_form';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // We will need these values inside preprocess.
    $session = $this->getRequest()->getSession();
    $session->set('beehotel_requested_range', $form_state->getValue('range'));
  }

  /**
   * List of timerange for vertical.
   *
   *  @todo add coming months.
   */
  public function timeRanges() {

    $ranges = [
      'cw' => [
        'label' => t("Current week"),
        'value' => "",
        'rows' => "7",
      ],

      'cm' => [
        'label' => t("Current month"),
        'value' => "",
        'rows' => date("t"),
      ],

      'n15d' => [
        'label' => t("Next 15 days"),
        'value' => "",
        'rows' => 15,
      ],

      'n30d' => [
        'label' => t("Next 30 days"),
        'rows' => 30,
      ],

      'n45d' => [
        'label' => t("Next 45 days"),
        'rows' => 45,
      ],

      'n60d' => [
        'label' => t("Next 60 days"),
        'rows' => 60,
      ],
      'n180d' => [
        'label' => t("Next 180 days"),
        'rows' => 180,
      ],
    ];
    return $ranges;
  }

}
